DROP PROCEDURE [dbo].[RemoveLongItem_Hekaton]
GO
DROP PROCEDURE [dbo].[GetAllLongItemParts_Hekaton]
GO
DROP PROCEDURE [dbo].[GetLongItemPart_Hekaton]
GO
DROP TABLE [dbo].[LongItemTable]
GO

CREATE TABLE [dbo].[LongItemTable] 
(
       [ItemID] [nvarchar](88) collate Latin1_General_100_BIN2 NOT NULL index idx_hash hash with (bucket_count = 10000000),
       [TotalLength] [int] NOT NULL,
	   [ItemPart] [int] NOT NULL,
	   [ItemLong] [varbinary](7000) NOT NULL,
	   CONSTRAINT [pk_LongItemTable] PRIMARY KEY NONCLUSTERED HASH ([ItemID], [ItemPart]) WITH(BUCKET_COUNT = 10000000)
)
WITH (MEMORY_OPTIMIZED = ON, 
--durability = SCHEMA_AND_DATA) 
durability = SCHEMA_ONLY) 
GO

CREATE PROCEDURE [dbo].[RemoveLongItem_Hekaton]
	@Id nvarchar(88)
WITH NATIVE_COMPILATION, SCHEMABINDING, EXECUTE AS OWNER AS
BEGIN ATOMIC WITH (TRANSACTION ISOLATION LEVEL = SNAPSHOT, LANGUAGE = 'English')
	DELETE FROM dbo.LongItemTable 
	WHERE ItemID=@ID
END
GO

CREATE PROCEDURE [dbo].[GetAllLongItemParts_Hekaton]
	@Id nvarchar(88)
WITH NATIVE_COMPILATION, SCHEMABINDING, EXECUTE AS OWNER AS
BEGIN ATOMIC WITH (TRANSACTION ISOLATION LEVEL = SNAPSHOT, LANGUAGE = 'English')
	SELECT ItemID, TotalLength, ItemPart, ItemLong FROM dbo.LongItemTable
	WHERE ItemID=@ID
END
GO

CREATE PROCEDURE [dbo].[GetLongItemPart_Hekaton]
			@Id nvarchar(88),
			@PartNo int,
			@ThePart varbinary(7000) OUTPUT,
			@IsPartAvailable bit OUTPUT
WITH NATIVE_COMPILATION, SCHEMABINDING, EXECUTE AS OWNER AS
BEGIN ATOMIC WITH (TRANSACTION ISOLATION LEVEL = SNAPSHOT, LANGUAGE = 'English')
	DECLARE @Count int=0
	SELECT @Count+=1 FROM dbo.LongItemTable
	WHERE
		ItemID=@ID
		AND ItemPart=@PartNo
	
	IF @Count=0
		SET @IsPartAvailable=0
	ELSE
		SELECT @ThePart=ItemLong, @IsPartAvailable=1
		FROM dbo.LongItemTable
		WHERE ItemID=@Id AND ItemPart=@PartNo
END
GO


DROP PROCEDURE [dbo].[RetrieveBLOB]
GO
CREATE PROCEDURE [dbo].[RetrieveBLOB]
	@id nvarchar(88),
	@TheBlob varbinary(max) OUTPUT
AS
BEGIN
	DECLARE @Part int=0
	DECLARE @DoesPartExist bit=0
	DECLARE @Chunk varbinary(7000)=0x01
	
	-- Set output variable to NULL, so NULL gets returned if the item does not exist
	SET @TheBlob=NULL

	exec GetLongItemPart_Hekaton
		@id,
		@Part,
		@Chunk OUTPUT,
		@DoesPartExist OUTPUT
		
	SET @Part=7000
	if @DoesPartExist=1
		SET @TheBlob=convert(varbinary(max), @Chunk)
	
	WHILE @DoesPartExist=1
	BEGIN
		exec GetLongItemPart_Hekaton
			@id,
			@Part,
			@Chunk OUTPUT,
			@DoesPartExist OUTPUT
		SET @Part+=7000
		if @DoesPartExist=1
			SET @TheBlob=@TheBlob+convert(varbinary(max), @Chunk)
	END
END
GO

DROP PROCEDURE [dbo].[StoreBLOB]
GO
CREATE PROCEDURE [dbo].[StoreBLOB]
	@id nvarchar(88),
	@TheBlob varbinary(max)
AS
BEGIN
	DECLARE @Part int=0
	DECLARE @Chunk varbinary(7000)
	DECLARE @DL bigint=DATALENGTH(@TheBlob)
	WHILE @Part<@DL
	BEGIN
		SET @Chunk=SUBSTRING(@TheBlob, @Part+1, 7000)
		INSERT INTO LongItemTable
		VALUES 
		(
			@Id,
			@DL,
			@Part,
			@Chunk
		)
		SET @Part=@Part+7000
	END
END
GO

DROP PROCEDURE [dbo].[UpdateBLOB]
GO
CREATE PROCEDURE [dbo].[UpdateBLOB]
	@id nvarchar(88),
	@TheBlob varbinary(max)
AS
BEGIN
	EXEC RemoveLongItem_Hekaton @Id

	EXEC StoreBLOB @Id, @TheBlob	
END
GO
